"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var database_1 = require("../objects/database");
var DatabaseCommands = /** @class */ (function () {
    function DatabaseCommands(protocol) {
        this.protocol = protocol;
    }
    /**
     * Counts the number of songs and their total playtime in the database that match exactly.
     * `filter` is either a filter expression as described
     * [here](https://www.musicpd.org/doc/html/protocol.html#filter-syntax)
     * or an array of tuples containing a tag and the value that it should match.
     * Note that tags are case sensitive and that the MPD documentation incorrectly lists all
     * tags as lower-case. Use `mpc.reflection.tagTypes()` to get the correct list of tags
     * supported by MPD.
     */
    DatabaseCommands.prototype.count = function (filter) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'count';
                        cmd = addFilter(cmd, filter);
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, this.protocol.parse(lines, [], function (valueMap) { return new database_1.SongCount(valueMap); })[0]];
                }
            });
        });
    };
    /**
     * Counts the number of songs and their total playtime in the database that match exactly.
     * `filter` is either a filter expression as described
     * [here](https://www.musicpd.org/doc/html/protocol.html#filter-syntax)
     * or an array of tuples containing a tag and the value that it should match.
     * The results are grouped by tag `groupingTag` (e.g. 'Artist', 'Album', 'Date', 'Genre')
     * Note that tags are case sensitive and that the MPD documentation incorrectly lists all
     * tags as lower-case. Use `mpc.reflection.tagTypes()` to get the correct list of tags
     * supported by MPD.
     */
    DatabaseCommands.prototype.countGrouped = function (filter, groupingTag) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'count';
                        cmd = addFilter(cmd, filter);
                        cmd += " group " + groupingTag;
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, this.protocol.parse(lines, [groupingTag], function (valueMap) { return new database_1.GroupedSongCount(valueMap, groupingTag); })];
                }
            });
        });
    };
    /**
     * Finds songs in the database that match exactly.
     * `start` and `end` can be used to query only a portion of the real response.
     * `sort` sorts the result by the specified tag. The sort is descending if the tag is prefixed with a
     * minus (‘-‘). Without sort, the order is undefined. Only the first tag value will be used,
     * if multiple of the same type exist. To sort by “Artist”, “Album” or “AlbumArtist”, you should
     * specify “ArtistSort”, “AlbumSort” or “AlbumArtistSort” instead. These will automatically fall back
     * to the former if “*Sort” doesn’t exist. “AlbumArtist” falls back to just “Artist”. The type
     * “Last-Modified” can sort by file modification time.
     * `filter` is either a filter expression as described
     * [here](https://www.musicpd.org/doc/html/protocol.html#filter-syntax)
     * or an array of tuples containing a tag or one of the special parameters listed below
     * and the value that it should match. Supported special parameters are:
     * * 'any' checks all tag values
     * * 'file' checks the full path (relative to the music directory)
     * * 'base' restricts the search to songs in the given directory (also relative to the music directory)
     * * 'modified-since' compares the file's time stamp with the given value (ISO 8601 or UNIX time stamp)
     * Note that tags are case sensitive and that the MPD documentation incorrectly lists all
     * tags as lower-case. Use `mpc.reflection.tagTypes()` to get the correct list of tags
     * supported by MPD.
     */
    DatabaseCommands.prototype.find = function (filter, start, end, sort) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'find';
                        cmd = addFilter(cmd, filter);
                        cmd = addSort(cmd, sort);
                        cmd = addWindow(cmd, start, end);
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, this.protocol.parse(lines, ['file'], function (valueMap) { return database_1.DirectoryEntry.fromValueMap(valueMap, true); })];
                }
            });
        });
    };
    /**
     * Finds songs in the database that match exactly and adds them to the current playlist.
     * Parameters have the same meaning as for `find()`.
     */
    DatabaseCommands.prototype.findAdd = function (filter, start, end, sort) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'findadd';
                        cmd = addFilter(cmd, filter);
                        cmd = addSort(cmd, sort);
                        cmd = addWindow(cmd, start, end);
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Searches for any song that matches. Parameters have the same meaning as for `find()`,
     * except that the search is a case insensitive substring search.
     */
    DatabaseCommands.prototype.search = function (filter, start, end, sort) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'search';
                        cmd = addFilter(cmd, filter);
                        cmd = addSort(cmd, sort);
                        cmd = addWindow(cmd, start, end);
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, this.protocol.parse(lines, ['file'], function (valueMap) { return database_1.DirectoryEntry.fromValueMap(valueMap, true); })];
                }
            });
        });
    };
    /**
     * Searches for any song that matches and adds them to the current playlist.
     * Parameters have the same meaning as for `find()`, except that the search is a
     * case insensitive substring search.
     */
    DatabaseCommands.prototype.searchAdd = function (filter, start, end, sort) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'searchadd';
                        cmd = addFilter(cmd, filter);
                        cmd = addSort(cmd, sort);
                        cmd = addWindow(cmd, start, end);
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Searches for any song that matches and adds them to the playlist named `name`.
     * If a playlist by that name doesn't exist it is created.
     * Parameters have the same meaning as for `find()`, except that the search is a
     * case insensitive substring search.
     */
    DatabaseCommands.prototype.searchAddPlaylist = function (name, filter, start, end, sort) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = "searchaddpl " + name;
                        cmd = addFilter(cmd, filter);
                        cmd = addSort(cmd, sort);
                        cmd = addWindow(cmd, start, end);
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Lists the contents of the directory `uri`, including files are not recognized by MPD.
     * `uri` can be a path relative to the music directory or an URI understood by one of the
     * storage plugins. For example, "smb://SERVER" returns a list of all shares on the given
     * SMB/CIFS server; "nfs://servername/path" obtains a directory listing from the NFS server.
     */
    DatabaseCommands.prototype.listFiles = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'listfiles';
                        if (uri) {
                            cmd += " \"" + uri + "\"";
                        }
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, this.protocol.parse(lines, ['file', 'directory'], function (valueMap) { return database_1.DirectoryEntry.fromValueMap(valueMap, false); })];
                }
            });
        });
    };
    /**
     * Lists the contents of the directory `uri`. When listing the root directory, this currently
     * returns the list of stored playlists. This behavior is deprecated; use `listPlaylists()`
     * instead. This command may be used to list metadata of remote files (e.g. `uri` beginning
     * with "http://" or "smb://"). Clients that are connected via UNIX domain socket may use this
     * command to read the tags of an arbitrary local file (`uri` is an absolute path).
     */
    DatabaseCommands.prototype.listInfo = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'lsinfo';
                        if (uri) {
                            cmd += " \"" + uri + "\"";
                        }
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, this.protocol.parse(lines, ['file', 'playlist', 'directory'], function (valueMap) { return database_1.DirectoryEntry.fromValueMap(valueMap, true); })];
                }
            });
        });
    };
    /**
     * Lists all songs and directories in `uri` recursively. Do not use this command to manage a
     * client-side copy of MPD's database. That is fragile and adds huge overhead.
     * It will break with large databases. Instead, query MPD whenever you need something.
     */
    DatabaseCommands.prototype.listAll = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'listall';
                        if (uri) {
                            cmd += " \"" + uri + "\"";
                        }
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, lines.map(function (line) { return line.substring(line.indexOf(':') + 2); })];
                }
            });
        });
    };
    /**
     * Same as `listAll()`, except it also returns metadata info. Do not use this command to
     * manage a client-side copy of MPD's database. That is fragile and adds huge overhead.
     * It will break with large databases. Instead, query MPD whenever you need something.
     */
    DatabaseCommands.prototype.listAllInfo = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'listallinfo';
                        if (uri) {
                            cmd += " \"" + uri + "\"";
                        }
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, this.protocol.parse(lines, ['file', 'playlist', 'directory'], function (valueMap) { return database_1.DirectoryEntry.fromValueMap(valueMap, true); })];
                }
            });
        });
    };
    /**
     * Lists unique tags values of the specified type. `type` can be any tag supported by MPD
     * or 'file', but 'file' is deprecated. `filter` specifies a filter like the one in `find()`.
     * `groupingTags` may be used to group the results by one or more tags.
     * Note that tags are case sensitive and that the MPD documentation incorrectly lists all
     * tags as lower-case. Use `mpc.reflection.tagTypes()` to get the correct list of tags
     * supported by MPD.
     */
    DatabaseCommands.prototype.list = function (type, filter, groupingTags) {
        if (filter === void 0) { filter = []; }
        if (groupingTags === void 0) { groupingTags = []; }
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines, tagsGroupedByString, groupedTags;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = "list " + type;
                        cmd = addFilter(cmd, filter);
                        groupingTags.forEach(function (tag) {
                            cmd += " group " + tag;
                        });
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        tagsGroupedByString = new Map();
                        this.protocol.parse(lines, [type], function (map) {
                            var group = [];
                            groupingTags.forEach(function (groupingTag) { return group.push(map.get(groupingTag) || ''); });
                            var groupString = JSON.stringify(group);
                            if (!tagsGroupedByString.has(groupString)) {
                                tagsGroupedByString.set(groupString, []);
                            }
                            if (map.has(type)) {
                                tagsGroupedByString.get(groupString).push((map.get(type)));
                            }
                        });
                        groupedTags = new Map();
                        tagsGroupedByString.forEach(function (tags, groupString_1) {
                            var group_1 = JSON.parse(groupString_1);
                            groupedTags.set(group_1, tags);
                        });
                        return [2 /*return*/, groupedTags];
                }
            });
        });
    };
    /**
     * Read "comments" (i.e. key-value pairs) from the file specified by `uri`. This `uri` can be
     * a path relative to the music directory or an absolute path. This command may be used to list
     * metadata of remote files (e.g. `uri` beginning with "http://" or "smb://").
     * Comments with suspicious characters (e.g. newlines) are ignored silently.
     * The meaning of these depends on the codec, and not all decoder plugins support it.
     * For example, on Ogg files, this lists the Vorbis comments.
     */
    DatabaseCommands.prototype.readComments = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = "readcomments \"" + uri + "\"";
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, this.protocol.parse(lines, [], function (valueMap) { return valueMap; })[0]];
                }
            });
        });
    };
    /**
     * Updates the music database: find new files, remove deleted files, update modified files.
     * `uri` is a particular directory or song/file to update. If you do not specify it, everything
     * is updated. Returns a positive number identifying the update job. You can read the current
     * job id in the status response.
     */
    DatabaseCommands.prototype.update = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'update';
                        if (uri) {
                            cmd += " \"" + uri + "\"";
                        }
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, Number(lines[0].substring(13))];
                }
            });
        });
    };
    /**
     * Same as `update()`, but also rescans unmodified files.
     */
    DatabaseCommands.prototype.rescan = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var cmd, lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cmd = 'rescan';
                        if (uri) {
                            cmd += " \"" + uri + "\"";
                        }
                        return [4 /*yield*/, this.protocol.sendCommand(cmd)];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, Number(lines[0].substring(13))];
                }
            });
        });
    };
    return DatabaseCommands;
}());
exports.DatabaseCommands = DatabaseCommands;
function addFilter(cmd, filter) {
    if (typeof filter === 'string') {
        cmd += " \"" + filter.replace(/\\/g, '\\\\').replace(/\"/g, '\\\"') + "\"";
    }
    else {
        filter.forEach(function (tagAndNeedle) {
            cmd += " " + tagAndNeedle[0] + " \"" + tagAndNeedle[1] + "\"";
        });
    }
    return cmd;
}
function addSort(cmd, sort) {
    if (sort !== undefined) {
        cmd += " sort " + sort;
    }
    return cmd;
}
function addWindow(cmd, start, end) {
    if (start !== undefined) {
        cmd += " window " + start + ":" + ((end !== undefined) ? end : '');
    }
    return cmd;
}
//# sourceMappingURL=database.js.map