"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var decoder_1 = require("../objects/decoder");
var util_1 = require("../util");
var connection_1 = require("./connection");
var ReflectionCommands = /** @class */ (function () {
    function ReflectionCommands(protocol) {
        this.protocol = protocol;
    }
    /**
     * Dumps configuration values that may be interesting for the client.
     * This command is only permitted to "local" clients (connected via UNIX domain socket).
     * The following response attributes are available:
     * * `music_directory`: The absolute path of the music directory.
     */
    ReflectionCommands.prototype.config = function () {
        return __awaiter(this, void 0, void 0, function () {
            var lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.protocol.sendCommand('config')];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, this.protocol.parse(lines, [], function (valueMap) { return valueMap; })[0]];
                }
            });
        });
    };
    /**
     * Shows which commands the current user has access to.
     */
    ReflectionCommands.prototype.commands = function () {
        return __awaiter(this, void 0, void 0, function () {
            var lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.protocol.sendCommand('commands')];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, lines.map(function (line) { return line.substring(9); })];
                }
            });
        });
    };
    /**
     * Shows which commands the current user has access to.
     */
    ReflectionCommands.prototype.notCommands = function () {
        return __awaiter(this, void 0, void 0, function () {
            var lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.protocol.sendCommand('notcommands')];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, lines.map(function (line) { return line.substring(9); })];
                }
            });
        });
    };
    /**
     * This method is identical to the method by the same name in the ConnectionCommands class.
     * It is only provided here for backward compatibility.
     */
    ReflectionCommands.prototype.tagTypes = function () {
        return connection_1.tagTypes(this.protocol);
    };
    /**
     * Gets a list of available URL handlers.
     */
    ReflectionCommands.prototype.urlHandlers = function () {
        return __awaiter(this, void 0, void 0, function () {
            var lines;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.protocol.sendCommand('urlhandlers')];
                    case 1:
                        lines = _a.sent();
                        return [2 /*return*/, lines.map(function (line) { return line.substring(9); })];
                }
            });
        });
    };
    /**
     * Returns a list of decoder plugins with their supported suffixes and MIME types.
     */
    ReflectionCommands.prototype.decoders = function () {
        return __awaiter(this, void 0, void 0, function () {
            var lines, decoders, currentDecoder;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.protocol.sendCommand('decoders')];
                    case 1:
                        lines = _a.sent();
                        decoders = [];
                        lines.forEach(function (line) {
                            if (util_1.stringStartsWith(line, 'plugin')) {
                                if (currentDecoder) {
                                    decoders.push(currentDecoder);
                                }
                                currentDecoder = new decoder_1.Decoder(line.substring(8));
                            }
                            else if (util_1.stringStartsWith(line, 'suffix')) {
                                currentDecoder.suffixes.push(line.substr(8));
                            }
                            else if (util_1.stringStartsWith(line, 'mime_type')) {
                                currentDecoder.mimeTypes.push(line.substr(11));
                            }
                        });
                        return [2 /*return*/, decoders];
                }
            });
        });
    };
    return ReflectionCommands;
}());
exports.ReflectionCommands = ReflectionCommands;
//# sourceMappingURL=reflection.js.map