"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("../util");
/**
 * Base class for objects in the music database.
 */
var DirectoryEntry = /** @class */ (function () {
    function DirectoryEntry(valueMap, pathKey, entryType) {
        if (!valueMap.has(pathKey))
            throw new Error('Path not found for DirectoryEntry object');
        this.entryType = entryType;
        this.path = valueMap.get(pathKey);
        this.lastModified = util_1.getOptionalDate(valueMap, 'Last-Modified');
    }
    DirectoryEntry.fromValueMap = function (valueMap, withMetadata) {
        if (withMetadata === void 0) { withMetadata = true; }
        if (valueMap.get('file')) {
            if (withMetadata) {
                return new Song(valueMap);
            }
            else {
                return new File(valueMap);
            }
        }
        else if (valueMap.get('directory')) {
            return new Directory(valueMap);
        }
        else if (valueMap.get('playlist')) {
            return new Playlist(valueMap);
        }
        else {
            var keys = __spread(valueMap.keys()).map(function (key) { return "'" + key + "'"; }).join(', ');
            throw new Error("Couldn't determine type of directory entry with keys " + keys);
        }
    };
    DirectoryEntry.prototype.isFile = function () { return this.entryType === 'file'; };
    DirectoryEntry.prototype.isSong = function () { return this.entryType === 'song'; };
    DirectoryEntry.prototype.isPlaylist = function () { return this.entryType === 'playlist'; };
    DirectoryEntry.prototype.isDirectory = function () { return this.entryType === 'directory'; };
    return DirectoryEntry;
}());
exports.DirectoryEntry = DirectoryEntry;
var File = /** @class */ (function (_super) {
    __extends(File, _super);
    function File(valueMap) {
        var _this = _super.call(this, valueMap, 'file', 'file') || this;
        _this.size = valueMap.has('size') ? Number(valueMap.get('size')) : undefined;
        return _this;
    }
    return File;
}(DirectoryEntry));
exports.File = File;
var Song = /** @class */ (function (_super) {
    __extends(Song, _super);
    function Song(valueMap) {
        var _this = _super.call(this, valueMap, 'file', 'song') || this;
        _this.title = valueMap.get('Title');
        _this.name = valueMap.get('Name');
        _this.artist = valueMap.get('Artist');
        _this.artistSort = valueMap.get('ArtistSort');
        _this.composer = valueMap.get('Composer');
        _this.performer = valueMap.get('Performer');
        _this.album = valueMap.get('Album');
        _this.albumSort = valueMap.get('AlbumSort');
        _this.albumArtist = valueMap.get('AlbumArtist');
        _this.albumArtistSort = valueMap.get('AlbumArtistSort');
        _this.track = valueMap.get('Track');
        _this.disc = valueMap.get('Disc');
        _this.label = valueMap.get('Label');
        _this.date = valueMap.get('Date');
        _this.originalDate = valueMap.get('OriginalDate');
        _this.genre = valueMap.get('Genre');
        _this.comment = valueMap.get('Comment');
        _this.musicBrainzArtistId = valueMap.get('MUSICBRAINZ_ARTISTID');
        _this.musicBrainzAlbumId = valueMap.get('MUSICBRAINZ_ALBUMID');
        _this.musicBrainzAlbumArtistId = valueMap.get('MUSICBRAINZ_ALBUMARTISTID');
        _this.musicBrainzTrackId = valueMap.get('MUSICBRAINZ_TRACKID');
        _this.musicBrainzReleaseTrackId = valueMap.get('MUSICBRAINZ_RELEASETRACKID');
        _this.musicBrainzWorkId = valueMap.get('MUSICBRAINZ_WORKID');
        var durationString = valueMap.get('duration') || valueMap.get('Time');
        _this.duration = durationString ? Number(durationString) : undefined;
        _this.format = valueMap.get('format');
        var splitFormat = _this.format ? _this.format.split(':') : [];
        _this.sampleRate = util_1.parseOptionalNumber(splitFormat[0]);
        _this.bitDepth = util_1.parseOptionalNumber(splitFormat[1]);
        _this.channels = util_1.parseOptionalNumber(splitFormat[2]);
        return _this;
    }
    return Song;
}(DirectoryEntry));
exports.Song = Song;
var Playlist = /** @class */ (function (_super) {
    __extends(Playlist, _super);
    function Playlist(valueMap) {
        return _super.call(this, valueMap, 'playlist', 'playlist') || this;
    }
    return Playlist;
}(DirectoryEntry));
exports.Playlist = Playlist;
var Directory = /** @class */ (function (_super) {
    __extends(Directory, _super);
    function Directory(valueMap) {
        return _super.call(this, valueMap, 'directory', 'directory') || this;
    }
    return Directory;
}(DirectoryEntry));
exports.Directory = Directory;
var SongCount = /** @class */ (function () {
    function SongCount(valueMap) {
        if (!valueMap.has('songs'))
            throw new Error('Number of songs not found for SongCount object');
        if (!valueMap.has('playtime'))
            throw new Error('Playtime of songs not found for SongCount object');
        this.songs = Number(valueMap.get('songs'));
        this.playtime = Number(valueMap.get('playtime'));
    }
    return SongCount;
}());
exports.SongCount = SongCount;
var GroupedSongCount = /** @class */ (function (_super) {
    __extends(GroupedSongCount, _super);
    function GroupedSongCount(valueMap, groupingTag) {
        var _this = _super.call(this, valueMap) || this;
        if (!valueMap.has(groupingTag))
            throw new Error("'" + groupingTag + "' not found for GroupedSongCount object");
        _this.group = valueMap.get(groupingTag);
        return _this;
    }
    return GroupedSongCount;
}(SongCount));
exports.GroupedSongCount = GroupedSongCount;
//# sourceMappingURL=database.js.map